version 15
set matastrict on

mata:

/*      _
        N = n_choose_k(n, k)
                       -  -

                N = n! / ( (n-k)! k! )

        Formula evaluated using canceling to increase domain.
*/

real scalar n_choose_k(real scalar n, real scalar k)
{
        return( n-k > k ? 
                nfactorial_over_kfactorial(n, n-k) / 
                nfactorial_over_kfactorial(k, 1)
                :
                nfactorial_over_kfactorial(n, k) / 
                nfactorial_over_kfactorial(n-k, 1)
              )
}

/*      ______
        result = nfactorial_over_kfactorial(n, k)
                                            -  -
                
        Returns n!/k! calculated as n*(n-1)*...*(k+1).
        Note: 0! = 1.
*/


real scalar nfactorial_over_kfactorial(real scalar n, real scalar k)
{
        real scalar     result, i

        if (n<0 | n>1.0x+35 | n!=trunc(n))  return(.)
        if (k<0 | k>1.0x+35 | k!=trunc(k))  return(.)

        result = 1 
        for (i=n; i>k; --i) result = result*i

        return(result)
}
end


mata:

// Test 1
//   For small n, n_choose_k(n, k) should produce the same result 
//   as directly calculating n!/((n-k)!k!).

real scalar nk(real scalar n, real scalar k)
		return(factorial(n) / (factorial(n-k)*factorial(k)))

for (n=1; n<=5; n++) {
	for (k=0; k<=n+1; k++) {
		/* printf("Ch(%f, %f) = %f\n", n, k, n_choose_k(n,k)) */
		assert(n_choose_k(n, k) == nk(n, k))
	}
}

assert(n_choose_k(5,0) == 1)
assert(n_choose_k(5,6) == .)

// Test 2
//   n_choose_k() should produce results when direct calculation
//   fails.

assert(n_choose_k(1000,    1) == 1000)
assert(n_choose_k(1000,    2) == 1000*999/2)
assert(n_choose_k(1000, 1000) ==    1)
assert(n_choose_k(1000,  999) == 1000)
assert(n_choose_k(1000,  998) == n_choose_k(1000, 2))
assert(        nk(1000,    1) ==   .)   // sic, FYI
assert(        nk(1000, 1000) ==   .)   // sic, FYI

// Test 3
//   Missing values.

assert(n_choose_k(-1,  1)==.)
assert(n_choose_k( 1, -1)==.)
assert(n_choose_k( 5,  .)==.)
assert(n_choose_k( .,  2)==.)
assert(n_choose_k( .,  .)==.)

end
