// version number intentionally omitted

clear all
run lr1.mata

sysuse auto 
keep mpg weight displacement
* keep in 1/10

// ---------------------------------------- test 1, with intercept
reg mpg weight displacement 
predict double yhat

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 

r = lrset(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = lr_b(r)
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
lr_ee(r),  truee
mreldif(lr_ee(r), truee)
assert(mreldif(lr_ee(r), truee) < 1e-12)

// lr_s2
trus2 = st_numscalar("e(rmse)")^2
lr_s2(r), trus2

// lr_V
truV = st_matrix("e(V)") 
lr_V(r)
mreldif(lr_V(r), truV)
assert(mreldif(lr_V(r), truV) < 1e-12)

// lr_yhat
st_view(truyhat = ., ., "yhat")
mreldif(lr_yhat(r), truyhat)
assert(mreldif(lr_yhat(r), truyhat) <= 1e-13)

end


// ---------------------------------------- test 2, without intercept
reg mpg weight displacement, nocons
drop yhat 
predict double yhat 

mata:
st_view(X = ., ., ("weight", "displacement"))
st_view(y = ., ., "mpg") 

r = lrset(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
trub
b = lr_b(r)
b
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
lr_ee(r),  truee
mreldif(lr_ee(r), truee)
assert(mreldif(lr_ee(r), truee) < 1e-14)

// lr_s2
trus2 = st_numscalar("e(rmse)")^2
lr_s2(r), trus2

// lr_V
truV = st_matrix("e(V)") 
lr_V(r)
mreldif(lr_V(r), truV)
assert(mreldif(lr_V(r), truV) < 1e-14)

// lr_yhat
st_view(truyhat = ., ., "yhat")
mreldif(lr_yhat(r), truyhat)
assert(mreldif(lr_yhat(r), truyhat) <= 1e-13)
end


// ---------------------------------------- test 3, 0 RHS with intercept
reg mpg 
drop yhat 
predict double yhat

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)		// no rhs vars

r = lrset(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = lr_b(r)
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
lr_ee(r),  truee
mreldif(lr_ee(r), truee)
assert(mreldif(lr_ee(r), truee) < 1e-12)

// lr_s2
trus2 = st_numscalar("e(rmse)")^2
lr_s2(r), trus2

// lr_V
truV = st_matrix("e(V)") 
lr_V(r)
mreldif(lr_V(r), truV)
assert(mreldif(lr_V(r), truV) < 1e-12)

// lr_yhat
st_view(truyhat = ., ., "yhat")
mreldif(lr_yhat(r), truyhat)
assert(mreldif(lr_yhat(r), truyhat) == 0)
end

// ---------------------------------------- test 4, 0 RHS w/o intercept
reg mpg,  nocons
drop yhat 
predict double yhat 

mata:
st_view(y = ., ., "mpg") 
X = J(rows(y), 0, 0)		// no rhs vars

r = lrset(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
trub
b = lr_b(r)
b
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
lr_ee(r),  truee
mreldif(lr_ee(r), truee)
assert(mreldif(lr_ee(r), truee) < 1e-14)

// lr_s2
trus2 = st_numscalar("e(rmse)")^2
lr_s2(r), trus2

// lr_V
truV = st_matrix("e(V)") 
lr_V(r)
mreldif(lr_V(r), truV)
assert(mreldif(lr_V(r), truV) < 1e-14)

// lr_yhat
st_view(truyhat = ., ., "yhat")
mreldif(lr_yhat(r), truyhat)
assert(mreldif(lr_yhat(r), truyhat) == 0)
end



// ---------------------------------------- test 4, collinear w/ intercept
reg mpg weight displacement weight
drop yhat
predict double yhat 


mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r = lrset(y, X, 1)

// coefficients
trub = st_matrix("e(b)")
b = lr_b(r)
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
lr_ee(r),  truee
mreldif(lr_ee(r), truee)
assert(mreldif(lr_ee(r), truee) < 1e-12)

// lr_s2
trus2 = st_numscalar("e(rmse)")^2
lr_s2(r), trus2

// lr_V
truV = st_matrix("e(V)") 
lr_V(r)
mreldif(lr_V(r), truV)
assert(mreldif(lr_V(r), truV) < 1e-12)

// lr_yhat
st_view(truyhat = ., ., "yhat")
mreldif(lr_yhat(r), truyhat)
assert(mreldif(lr_yhat(r), truyhat) < 1e-13)

end


// ---------------------------------------- test 5, collinear w/o intercept

reg mpg weight displacement weight, nocons
drop yhat
predict double yhat 


mata:
st_view(X = ., ., ("weight", "displacement", "weight"))
st_view(y = ., ., "mpg") 

r = lrset(y, X, 0)

// coefficients
trub = st_matrix("e(b)")
b = lr_b(r)
b' \ trub
b'-trub
mreldif(b', trub) 
assert(mreldif(b', trub) < 1e-14)

// ess
truee = st_numscalar("e(rss)")
lr_ee(r),  truee
mreldif(lr_ee(r), truee)
assert(mreldif(lr_ee(r), truee) < 1e-12)

// lr_s2
trus2 = st_numscalar("e(rmse)")^2
lr_s2(r), trus2

// lr_V
truV = st_matrix("e(V)") 
lr_V(r)
mreldif(lr_V(r), truV)
assert(mreldif(lr_V(r), truV) < 1e-12)

// lr_yhat
st_view(truyhat = ., ., "yhat")
mreldif(lr_yhat(r), truyhat)
assert(mreldif(lr_yhat(r), truyhat) < 1e-13)

end


// ---------------------------------------- test 6, 0 obs w/ intercept

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r = lrset(y, X, 1)

// coefficients
b = lr_b(r)
b'
assert(b' == (0,0,0,.) | b'==(0,0,0,0))


// ess
assert(lr_ee(r)==0)

// lr_s2
assert(lr_s2(r)==0 | lr_s2(r)==.)

// lr_V
truV = J(3,3,0), J(3,1,0) \ J(1,3,0), .
lr_V(r)
assert(lr_V(r) == truV | lr_V(r)==J(4,4,.))

// lr_yhat
assert(rows(lr_yhat(r))==0 & cols(lr_yhat(r))==1)
end


// ---------------------------------------- test 7, 0 obs w/o intercept

mata:
X = J(0, 3, .)
y = J(0, 1, .) 

r = lrset(y, X, 0)

// coefficients
b = lr_b(r)
b'
assert(b' == (0,0,0))


// ess
assert(lr_ee(r)==0)

// lr_s2
assert(lr_s2(r)==.)

// lr_V
assert(lr_V(r) == J(3,3,.))

// lr_yhat
assert(rows(lr_yhat(r))==0 & cols(lr_yhat(r))==1)
end
